#ifndef UNIONFIND_H
#define UNIONFIND_H

#include <algorithm>

class Unionfind {
public:
  Unionfind(int size) {
    roots.resize(size);
    weights.resize(size);
    int n = 0;
    std::generate(roots.begin(), roots.end(), [&n] { return n++; });
    std::fill(weights.begin(), weights.end(), 1);
  }

  int find(int i) {
    int root_i = i;
    while (root_i != roots[root_i])
      root_i = roots[root_i];

    // path compression
    int cur = i;
    while (cur != root_i) {
      cur = roots[cur];
      roots[cur] = root_i;
    }

    return root_i;
  }

  void merge(int i, int j) {
    assert(i < (int)roots.size());
    assert(j < (int)roots.size());

    int root_i = find(i);
    int root_j = find(j);

    if (root_i == root_j)
      return;

    if (weights[root_i] > weights[root_j]) {
      roots[root_j] = root_i;
      weights[root_i] += weights[root_j];
    } else {
      roots[root_i] = root_j;
      weights[root_j] += weights[root_i];
    }
  }

private:
  std::vector<int> roots;
  std::vector<int> weights;
};

#endif // UNIONFIND_H
