#include "instance.hpp"
#include <cstdlib>
#include <cstring>

Instance*load_instance(const char*p){
	Instance*i = new Instance;
	FILE*f = fopen(p, "r");
	if(!f)
		return 0;
	if(!load_instance(f, *i)){
		delete i;
		return 0;
	}
	return i;
}

static const char header[] = "         x         y     right       top      left    bottom    text\n";

bool load_instance(std::FILE*f, Instance&i_){
	Instance i;

	// skip header
	int c;
	const char*h = header;
	while(*h){
		c = fgetc(f);
		if(c == EOF || c != *h)
			return false;
		++h;
	}

	Label l;
	const int buf_size = 1024*2;
	char buf[buf_size];
	while(fscanf(f, "%lf%lf%lf%lf%lf%lf",
		&l.pos.x, &l.pos.y, &l.right, 
		&l.top, &l.left, &l.bottom
		) == 6 
		&& fgets(buf, buf_size, f) == buf){
		char*s = buf;
		while(std::isspace(*s))
			++s;
		s[std::strlen(s)-1] = 0;
		l.text = s;
		i.add_label(l);
	}

	i_ = i;
	return true;
}

void save_instance(std::FILE*f, const Instance&i){
	fputs(header, f);
	for(unsigned j=0; j<i.count_labels(); ++j){
		const Label&l = i.get_label(j);
		fprintf(f, "%10.2lf%10.2lf%10.2lf%10.2lf%10.2lf%10.2lf    %s\n", 
			l.pos.x, l.pos.y, l.right, 
			l.top, l.left, l.bottom,
			l.text.c_str()
		);
	}
}

void save_instance(const char*n, const Instance&i){
	FILE*f = fopen(n, "w");
	save_instance(f, i);
	fclose(f);
}

static
double random_double(double min, double max){
	return static_cast<double>(std::rand())/RAND_MAX * (max - min + 1.0) + min;
}

Label random_label(){
	Label l;
	l.pos.x = random_double(-400, 400);
	l.pos.y = random_double(-300, 600);

	double w = random_double(50, 300);
	double h = random_double(10, 30);

	l.left = random_double(0, w);
	l.right = w-l.left;
	l.top = random_double(0, h);
	l.bottom = h-l.top;

	l.text = "random label";
	return l;
}

Instance random_instance(unsigned label_count){
	Instance i;
	while(label_count-- > 0)
		i.add_label(random_label());
	return i;
}


