#ifndef MIP_MODEL_NO_FLICKER_H
#define MIP_MODEL_NO_FLICKER_H

#include "mip_model.h"

#include "gurobi_c++.h"

namespace MIPModel{

void calcSolutionByMipNoFlickering(const AlgorithmInfo &algo, const Instance &inst, Solution &sol) {

    GRBEnv gurobi_env = MIPModel::set_gurobi_env(algo);
    GRBModel gurobi_model = GRBModel(gurobi_env);
    // Setze auf Maximierungsproblem
    gurobi_model.set(GRB_IntAttr_ModelSense, -1);

    vector<GRBVar> x;
    x.resize(inst.count_labels());

    /* add variables */
    for (unsigned int i = 0; i < inst.count_labels(); i++)
        x[i] = gurobi_model.addVar(0.0, 1.0, 1.0, GRB_BINARY);

    gurobi_model.update();

    // avoid overlaps
    const std::vector<conflict_information> &conflicts = inst.get_all_conflicts();
    for (const conflict_information &conflict : conflicts) {
        gurobi_model.addConstr(x[conflict.label_id1] + x[conflict.label_id2] <=
                1.0);
    }

    // hard conflicts
    if (algo.use_hard_conflicts) {
        for (unsigned int i = 0; i < inst.count_labels(); ++i) {
            if (inst.get_hard_conflicts(i) != AngleSet::empty)
                gurobi_model.addConstr(x[i] == 0);
        }
    }

    /* solve */
    gurobi_model.optimize();

    /* retranslate solution */
    for (unsigned int i = 0; i < inst.count_labels(); i++) {
        bool active = (x[i].get(GRB_DoubleAttr_X) == 1);

        if (active) {
            AngleSet sol_i(0, 1.9999999 * M_PI);
            sol.set_active(i, sol_i);
        }
    }
}

}


#endif // MIP_MODEL_NO_FLICKER_H
