#ifndef TOOLS_HPP
#define TOOLS_HPP

#include <sys/time.h>
#include <cassert>
#include <cmath>
#include <iostream>

class Timer
{
	public:
		Timer() {
			start = timestamp();
		} // postcondition: elapsed()==0
		void   restart() {
			start = timestamp();
		} // post: elapsed()==0

		double elapsed()                  // return elapsed time in ms
		{
			double cur = timestamp();
			return cur-start;
		}

	private:
		double start;

		/** Returns a timestamp ('now') in miliseconds (incl. a fractional part). */
		inline double timestamp() {
			timeval tp;
			gettimeofday(&tp, NULL);
			double ms = double(tp.tv_usec) / 1000.;
			return tp.tv_sec*1000 + ms;
		}
}; // timer


template <typename T>
void print_container(const T& container) {
    for (const auto& elem: container)
        std::cout << elem << " ";

    std::cout << std::endl;

}

#endif
